@@ -1,7 +1,8 @@
-import datajoint as dj
+import inspect
 import pathlib
 import importlib
-import inspect
+
+import datajoint as dj
 from element_interface.utils import find_root_directory, find_full_path
 
 schema = dj.schema()
@@ -12,32 +13,36 @@ _linking_module = None
 def activate(
     scan_schema_name, *, create_schema=True, create_tables=True, linking_module=None
 ):
-    """
-    activate(scan_schema_name, *, create_schema=True, create_tables=True, linking_module=None)
-        :param scan_schema_name: schema name on the database server to activate the `scan` module
-        :param create_schema: when True (default), create schema in the database if it does not yet exist.
-        :param create_tables: when True (default), create tables in the database if they do not yet exist.
-        :param linking_module: a module name or a module containing the
-         required dependencies to activate the `scan` module:
-            Upstream tables:
-                + Session: parent table to Scan, typically identifying a recording session
-                + Equipment: Reference table for Scan, specifying the equipment used for the acquisition of this scan
-                + Location: Reference table for ScanLocation, specifying the brain location where this scan is acquired
-            Functions:
-                + get_imaging_root_data_dir() -> list
-                    Retrieve the full path for the root data directory - e.g. containing the imaging recording files or analysis results for all subject/sessions.
-                    :return: a string (or list of string) for full path to the root data directory
-                + get_scan_image_files(scan_key: dict) -> list
-                    Retrieve the list of ScanImage files associated with a given Scan
-                    :param scan_key: key of a Scan
-                    :return: list of ScanImage files' full file-paths
-                + get_scan_box_files(scan_key: dict) -> list
-                    Retrieve the list of Scanbox files (*.sbx) associated with a given Scan
-                    :param scan_key: key of a Scan
-                    :return: list of Scanbox files' full file-paths
-                + get_processed_root_data_dir() -> str:
-                    Retrieves the root directory for all processed data to be found from or written to
-                    :return: a string for full path to the root directory for processed data
+    """Activate this schema.
+
+    Args:
+        scan_schema_name (str): schema name on the database server to activate the `scan` module
+        create_schema (bool): when True (default), create schema in the database if it does not yet exist.
+        create_tables (bool): when True (default), create tables in the database if they do not yet exist.
+        linking_module (str): a module name or a module containing the required dependencies to activate
+            the `scan` module.
+
+    Dependencies:
+    Upstream tables:
+        + Session: parent table to Scan, typically identifying a recording session
+        + Equipment: Reference table for Scan, specifying the equipment used for the acquisition of this scan
+        + Location: Reference table for ScanLocation, specifying the brain location where this scan is acquired
+
+    Functions:
+        + get_imaging_root_data_dir() -> list
+            Retrieve the full path for the root data directory - e.g. containing the imaging recording files or analysis results for all subject/sessions.
+            :return: a string (or list of string) for full path to the root data directory
+        + get_scan_image_files(scan_key: dict) -> list
+            Retrieve the list of ScanImage files associated with a given Scan
+            :param scan_key: key of a Scan
+            :return: list of ScanImage files' full file-paths
+        + get_scan_box_files(scan_key: dict) -> list
+            Retrieve the list of Scanbox files (*.sbx) associated with a given Scan
+            :param scan_key: key of a Scan
+            :return: list of Scanbox files' full file-paths
+        + get_processed_root_data_dir() -> str:
+            Retrieves the root directory for all processed data to be found from or written to
+            :return: a string for full path to the root directory for processed data
     """
 
     if isinstance(linking_module, str):
@@ -61,18 +66,16 @@ def activate(
 
 
 def get_imaging_root_data_dir() -> str:
-    """
-    All data paths, directories in DataJoint Elements are recommended to be stored as
-    relative paths (posix format), with respect to some user-configured "root" directory,
-     which varies from machine to machine (e.g. different mounted drive locations)
+    """Retrieve the root data directories containing the imaging data
+    for all subjects/sessions (e.g. acquired ScanImage raw files, output files from
+    processing routines, etc.). All data paths, directories in DataJoint Elements are
+    recommended to be stored as relative paths (posix format), with respect to some
+    user-configured "root" directory, which varies from machine to machine
+    (e.g. different mounted drive locations).
 
-    get_imaging_root_data_dir() -> list
-        This user-provided function retrieves the possible root data directories
-         containing the imaging data for all subjects/sessions
-         (e.g. acquired ScanImage raw files,
-         output files from processing routines, etc.)
-        :return: a string for full path to the imaging root data directory,
-         or list of strings for possible root data directories
+    Returns:
+        A string for full path to the imaging root data directory, or list of strings for
+        possible root data directories.
     """
 
     root_directories = _linking_module.get_imaging_root_data_dir()
@@ -86,10 +89,14 @@ def get_imaging_root_data_dir() -> str:
 
 
 def get_processed_root_data_dir() -> str:
-    """
-    get_processed_root_data_dir() -> str:
-        Retrieves the root directory for all processed data to be found from or written to
-        :return: a string for full path to the root directory for processed data
+    """Retrieve the root directory for all processed data. All data paths, directories
+    in DataJoint Elements are recommended to be stored as relative paths (posix format),
+    with respect to some user-configured "root" directory, which varies from machine to
+    machine (e.g. different mounted drive locations).
+
+
+    Returns:
+        A string for full path to the root directory for processed data.
     """
 
     if hasattr(_linking_module, "get_processed_root_data_dir"):
@@ -99,41 +106,64 @@ def get_processed_root_data_dir() -> str:
 
 
 def get_scan_image_files(scan_key: dict) -> list:
-    """
-    Retrieve the list of ScanImage files associated with a given Scan
-    :param scan_key: key of a Scan
-    :return: list of ScanImage files' full file-paths
+    """Retrieve the list of ScanImage files associated with a given Scan.
+
+    Args:
+        scan_key: Primary key of a Scan entry.
+
+    Returns:
+        A list of ScanImage files' full file-paths.
     """
     return _linking_module.get_scan_image_files(scan_key)
 
 
 def get_scan_box_files(scan_key: dict) -> list:
-    """
-    Retrieve the list of Scanbox files (*.sbx) associated with a given Scan
-    :param scan_key: key of a Scan
-    :return: list of Scanbox files' full file-paths
+    """Retrieve the list of Scanbox files (*.sbx) associated with a given Scan.
+
+    Args:
+        scan_key: Primary key of a Scan entry.
+
+    Returns:
+        A list of Scanbox files' full file-paths.
     """
     return _linking_module.get_scan_box_files(scan_key)
 
 
 def get_nd2_files(scan_key: dict) -> list:
-    """
-    Retrieve the list of Nikon files (*.nd2) associated with a given Scan
-    :param scan_key: key of a Scan
-    :return: list of Nikon files' full file-paths
+    """Retrieve the list of Nikon files (*.nd2) associated with a given Scan.
+
+    Args:
+        scan_key: Primary key of a Scan entry.
+
+    Returns:
+        A list of Nikon files' full file-paths.
     """
     return _linking_module.get_nd2_files(scan_key)
 
 
+def get_prairieview_files(scan_key: dict) -> list:
+    """Retrieve the list of Bruker PrairieView tif files (*.tif) with a given Scan
+
+    Args:
+        scan_key: Primary key of a Scan entry.
+
+    Returns:
+        A list of Bruker PrairieView files' full file-paths.
+    """
+    return _linking_module.get_prairieview_files(scan_key)
+
+
 # ----------------------------- Table declarations ----------------------
 
 
 @schema
 class AcquisitionSoftware(dj.Lookup):
-    definition = """  # Name of acquisition software - e.g. ScanImage, Scanbox, NIS
+    """A list of acquisition softwares supported by the workflow. Required to define a scan."""
+
+    definition = """  # Acquisition softwares
     acq_software: varchar(24)    
     """
-    contents = zip(["ScanImage", "Scanbox", "NIS"])
+    contents = zip(["ScanImage", "Scanbox", "NIS", "PrairieView"])
 
 
 @schema
@@ -146,6 +176,10 @@ class Channel(dj.Lookup):
 
 @schema
 class Scan(dj.Manual):
+    """A scan entry in this table is defined by a measurement done using a scanner (Equipment)
+    and an acquisition software. The details of the scanning data is placed in other tables,
+    including, ScanLocation, ScanInfo, and ScanInfo's part tables."""
+
     definition = """    
     -> Session
     scan_id: int        
@@ -158,16 +192,25 @@ class Scan(dj.Manual):
 
 @schema
 class ScanLocation(dj.Manual):
-    definition = """
+    """Anatomical location of the scanned region in the brain
+
+    Refer to the `definition` attribute for the table design."""
+
+    definition = """Anatomical location
     -> Scan   
-    ---    
+    ---
     -> Location      
     """
 
 
 @schema
 class ScanInfo(dj.Imported):
-    definition = """ # general data about the reso/meso scans from header
+    """
+    Information about the scan extracted from the recorded files.
+    Refer to the `definition` property to see the table design.
+    """
+
+    definition = """ # General data about the reso/meso scans from header
     -> Scan
     ---
     nfields              : tinyint   # number of fields
@@ -184,9 +227,12 @@ class ScanInfo(dj.Imported):
     fill_fraction=null   : float     # raster scan temporal fill fraction (see scanimage)
     scan_datetime=null   : datetime  # datetime of the scan
     scan_duration=null   : float     # (seconds) duration of the scan
+    bidirectional_z=null : boolean   # true = bidirectional z-scan
     """
 
     class Field(dj.Part):
+        """Stores field information of scan, including its coordinates, size, pixel pitch, etc."""
+
         definition = """ # field-specific scan information
         -> master
         field_idx         : int
@@ -203,9 +249,11 @@ class ScanInfo(dj.Imported):
         """
 
     class ScanFile(dj.Part):
+        """Filepath of the scan relative to root data directory"""
+
         definition = """
         -> master
-        file_path: varchar(255)  # filepath relative to root data directory
+        file_path: varchar(255)  # Filepath relative to root data directory
         """
 
     def make(self, key):
@@ -432,6 +480,45 @@ class ScanInfo(dj.Imported):
                         for plane_idx in range(nd2_file.sizes.get("Z", 1))
                     ]
                 )
+        elif acq_software == "PrairieView":
+            from element_interface import prairieviewreader
+
+            scan_filepaths = get_prairieview_files(key)
+            pvscan_info = prairieviewreader.get_pv_metadata(scan_filepaths[0])
+            self.insert1(
+                dict(
+                    key,
+                    nfields=pvscan_info["num_fields"],
+                    nchannels=pvscan_info["num_channels"],
+                    ndepths=pvscan_info["num_planes"],
+                    nframes=pvscan_info["num_frames"],
+                    nrois=pvscan_info["num_rois"],
+                    x=pvscan_info["x_pos"],
+                    y=pvscan_info["y_pos"],
+                    z=pvscan_info["z_pos"],
+                    fps=pvscan_info["frame_rate"],
+                    bidirectional=pvscan_info["bidirectional"],
+                    bidirectional_z=pvscan_info["bidirectional_z"],
+                    usecs_per_line=pvscan_info["usecs_per_line"],
+                    scan_datetime=pvscan_info["scan_datetime"],
+                    scan_duration=pvscan_info["scan_duration"],
+                )
+            )
+
+            self.Field.insert(
+                dict(
+                    key,
+                    field_idx=plane_idx,
+                    px_height=pvscan_info["height_in_pixels"],
+                    px_width=pvscan_info["width_in_pixels"],
+                    um_height=pvscan_info["height_in_um"],
+                    um_width=pvscan_info["width_in_um"],
+                    field_x=pvscan_info["fieldX"],
+                    field_y=pvscan_info["fieldY"],
+                    field_z=pvscan_info["fieldZ"][plane_idx],
+                )
+                for plane_idx in range(pvscan_info["num_planes"])
+            )
         else:
             raise NotImplementedError(
                 f"Loading routine not implemented for {acq_software} "
